defmodule DecentralisedBookIndex.Sync.DataTransformers.PublisherSyncTest do
  use DecentralisedBookIndex.DataCase, async: true

  alias DecentralisedBookIndex.Sync.PublisherSync
  alias DecentralisedBookIndex.Metadata

  alias DecentralisedBookIndex.TestEndpoints
  @test_server_endpoint TestEndpoints.test_api_endpoint()

  setup do
    user = generate(user(role: :moderator))
    %{user: user}
  end

  describe "sync publisher transformations" do
    test "a new publisher will be created" do
      server = generate(dbi_server(url: @test_server_endpoint))

      publisher = %{
        id: "11349865-1b7b-454a-b999-6c4059888a78",
        name: "Publisher",
        inserted_at: "2025-03-21T09:20:48.791539Z",
        updated_at: "2025-03-21T09:20:48.791539Z"
      }

      assert :ok = PublisherSync.create_update(publisher, server.id)
      assert {:ok, saved_publisher} = Metadata.get_publisher_by_id(publisher.id)

      publisher = datetime_from_iso8601_for_map(publisher)

      assert get_submap(saved_publisher, publisher) == publisher
      assert server.id == saved_publisher.dbi_server_id
    end

    test "update an existing publisher", %{user: user} do
      server = generate(dbi_server(url: @test_server_endpoint))

      {:ok, publisher} = Metadata.create_publisher("Publisher", actor: user)
      {:ok, publisher} = Metadata.assign_publisher_dbi_server(publisher, server.id, actor: user)

      publisher_attrs = %{
        id: publisher.id,
        name: "Publisher",
        inserted_at: "2025-03-21T09:20:48.791539Z",
        updated_at: datetime_add_second_as_string(publisher.updated_at, 10),
        dbi_server_id: server.id
      }

      assert :ok = PublisherSync.create_update(publisher_attrs, server.id)
      assert {:ok, saved_publisher} = Metadata.get_publisher_by_id(publisher.id)

      publisher_attrs = datetime_from_iso8601_for_map(publisher_attrs)

      assert get_submap(saved_publisher, publisher_attrs) == publisher_attrs
      assert server.id == saved_publisher.dbi_server_id
    end

    test "the local existing publisher wouldn't updated", %{user: user} do
      server = generate(dbi_server(url: @test_server_endpoint))

      {:ok, publisher} = Metadata.create_publisher("Publisher", actor: user)

      publisher_attrs = %{
        id: publisher.id,
        name: "Updated Publisher",
        inserted_at: "2025-03-21T09:20:48.791539Z",
        updated_at: "2025-03-21T09:20:48.791539Z"
      }

      assert :ok = PublisherSync.create_update(publisher_attrs, server.id)
      assert {:ok, saved_publisher} = Metadata.get_publisher_by_id(publisher.id)

      assert get_submap(saved_publisher, publisher_attrs) != publisher_attrs
      assert saved_publisher.name == "Publisher"
      assert saved_publisher.dbi_server_id == nil
    end

    test "old publisher wouldn't be sync", %{user: user} do
      server = generate(dbi_server(url: @test_server_endpoint))

      {:ok, publisher} = Metadata.create_publisher("Publisher", actor: user)
      {:ok, publisher} = Metadata.assign_publisher_dbi_server(publisher, server.id, actor: user)

      publisher_attrs = %{
        id: publisher.id,
        name: "Old Publisher",
        inserted_at: "2022-03-21T09:20:48.791539Z",
        updated_at: "2022-03-21T09:20:48.791539Z",
        dbi_server_id: server.id
      }

      assert :ok = PublisherSync.create_update(publisher_attrs, server.id)
      assert {:ok, saved_publisher} = Metadata.get_publisher_by_id(publisher.id)

      assert get_submap(saved_publisher, publisher_attrs) != publisher_attrs
      assert saved_publisher.name == "Publisher"
    end
  end
end
